((app) => {
        function MosaicPanelWidget() {
            const filterByCategory = 0;
            const filterByLabel = 1;
            const defaultOptions = {
                getTagGradientColor: function (tag) {
                    let isOpera = !!window.opera || navigator.userAgent.indexOf(' OPR/') >= 0;  // Opera 8.0+ (UA detection to detect Blink/v8-powered Opera)
                    let isFirefox = typeof InstallTrigger !== 'undefined';   // Firefox 1.0+
                    let isSafari = Object.prototype.toString.call(window.HTMLElement).indexOf('Constructor') > 0;   // At least Safari 3+: "[object HTMLElementConstructor]"
                    let isChrome = !!window.chrome && !isOpera;              // Chrome 1+
                    // var isIE = /*@cc_on!@*/false || !!document.documentMode; // At least IE6

                    if (isOpera) {
                        return "-o-linear-gradient(right bottom,  rgba(20,46,51,1) 0%, " + tag.color + " 100%)";
                    }
                    if (isFirefox) {
                        return "-moz-linear-gradient(right bottom,  rgba(20,46,51,1) 0%, " + tag.color + " 100%)";
                    }
                    if ((isSafari) || (isChrome)) {
                        return "-webkit-linear-gradient(right bottom,  rgba(20,46,51,1) 0%, " + tag.color + " 100%)";
                    }
                },
                getPreviewPath: function (tagOrDoc) {
                    // eslint-disable-next-line no-undef
                    return Utils.getPreviewPath(tagOrDoc);
                },
                htmlElement: function () {
                    return $('#mosaic_tab')
                },
                isOpen: false,
                pubsub: function () {
                    return app.rekall.Rekall('pubSub');
                },
                rekall: function () {
                    return app.rekall.Rekall('rekall');
                },
                filters_type_to_hide: [],
                filter_label_to_hide: [],
                _followedItems: [],
                _filter_type: filterByCategory,
            };

            const localOptions = $.extend({}, defaultOptions);

            // eslint-disable-next-line no-unused-vars
            localOptions._resetFilters = function (event) {
                $('.mosaic_filter_item')
                    .removeClass('mosaic_filter_disabled');
                $('.mosaic_label_filter_enabled').click();
                localOptions.filters_type_to_hide = [];
                localOptions.filter_label_to_hide = [];
                $('.mosaic_category').show();
            }

            localOptions._destroyFollowedElements = function () {
                let elementsToDestroy = localOptions._followedItems;
                localOptions._followedItems = [];
                elementsToDestroy.forEach(e => e.remove());
            }

            function toggleFilterType(filterName) {
                let filters = localOptions.filters_type_to_hide;
                if (filters.includes(filterName)) {
                    filters.splice(filters.indexOf(filterName), 1);
                } else {
                    filters.push(filterName);
                }
            }

            function toggleLabelToShow(filterName) {
                let filters = localOptions.filter_label_to_hide;
                if (filters.includes(filterName)) {
                    filters.splice(filters.indexOf(filterName), 1);
                } else {
                    filters.push(filterName);
                }
            }

            const refreshView = function () {
                emptyPanel();
                fillPanel();
            };

            const getUrl = function (tagOrDoc, path) {
                if ('undefined' === typeof path) {
                    let [reg, type] = tagOrDoc.getMetadata("Rekall->Type").split('/');
                    if (reg === 'rekall') {
                        if (type === 'marker') type = 'note';
                    }
                    return tagOrDoc.getDownloadLink() || "../shared/css/images/img-" + type + ".png";
                } else {
                    return path;
                }
            }

            const createFilterTypeSelector = function() {
                let selectDiv = $('<div class="filter-type-selector">');
                selectDiv.addClass("mosaic_filter");

                let outerDiv = $('<div class="mosaic_filter_item_select" style="display: flex; flex-direction: row; justify-content:center; align-items: center; align-content: center; ">');

                let selectElement = $('<select>');

                let createOptionElement =  function (value, text, isSelected){
                    let option = $('<option>');
                    option.attr('value', value);
                    option.text(text);
                    if (isSelected) {
                        option.attr('selected', 'selected');
                    }
                    return option;
                }

                selectElement.append(createOptionElement(0, "Filter by type of annotation document", 0 === localOptions._filter_type));
                selectElement.append(createOptionElement(1, "Filter by tag", 1 === localOptions._filter_type));
                selectElement.change(function(event) {
                    localOptions._filter_type = parseInt(selectElement.val());
                    refreshView();
                })

                outerDiv.append(selectElement)
                selectDiv.append(outerDiv);

                return selectDiv;
            }

            function rgbToHex(r, g, b) {
                return "#" + ((1 << 24) + (r << 16) + (g << 8) + b).toString(16).slice(1);
            }

            const createHtmlElementForTagOrDoc = function (tagOrDoc) {
                let path = localOptions.getPreviewPath(tagOrDoc);
                let name = tagOrDoc.getMetadata("Rekall->Name");
                let url = getUrl(tagOrDoc, path);

                let div = $('<div/>').addClass('mosaic_item mosaic_button').on('click', function () {
                    tagOrDoc.openPopupEdit();
                });
                let divImageContainer = $('<div>').addClass('mosaic_item_thumbnail_container');
                divImageContainer
                    .append($('<img src="'+url+'" alt="thumbnail image"/>')
                        .addClass('mosaic_item_thumbnail_container_img')
                        .attr('onerror', "this.src='../shared/css/images/img-document.png';"));
                div.append(divImageContainer);
                div.append($('<span/>')
                    .addClass('caption')
                    .text(name));

                let divUpdater = {
                    div: div,
                    tag: tagOrDoc
                };
                divUpdater.update = function (tag) {
                    // setDivVisibilityForTag(div, tag);
                }
                divUpdater.remove = function () {
                    tagOrDoc.removeObserver(divUpdater);
                }

                tagOrDoc.addObserver(divUpdater);
                localOptions._followedItems.push(divUpdater);

                return div;
            }

            const getFilterElement = function (text, color, callback, css_class, isToggled) {
                let result = $('<div/>')
                    .addClass('mosaic_filter_item mosaic_button')
                    .addClass(css_class)
                    .on('click', callback)
                    .css('background-color', color)
                    .append($('<h2/>').text(text));


                if (isToggled) {
                    result.toggleClass('mosaic_filter_disabled');
                }

                return result;
            }

            const emptyPanel = function () {
                localOptions.htmlElement().html('');
                localOptions._destroyFollowedElements();
            }

            function createViewByCategory(typefilterdiv) {
                for (let [k, v] of Object.entries(localOptions.rekall().sortings.colors.categories)) {
                    let categoryName = localOptions.rekall().sortings.colors.getCategoryName(k);
                    let category = $('<div/>').addClass('mosaic_category').css('background', localOptions.getTagGradientColor(v));

                    let grid = $('<div/>').addClass('mosaic_category_grid');

                    category.append($('<h2/>').text(categoryName));
                    category.append(grid);

                    typefilterdiv.append(getFilterElement(
                        categoryName,
                        v.color, function () {
                            category.toggle();
                            $(this).toggleClass('mosaic_filter_disabled');
                            toggleFilterType(categoryName)
                        },
                        '',
                        localOptions.filters_type_to_hide.includes(categoryName)
                    ));

                    localOptions.htmlElement().append(category);
                    if(localOptions.filters_type_to_hide.includes(categoryName)){
                        category.toggle();
                    }

                    for (let i in v.tags) {
                        grid.append(createHtmlElementForTagOrDoc(v.tags[i]));
                    }
                }
            }

            function createViewByLabel(typefilterdiv) {
                // retrieve labels with its tags
                const groupTagsByLabels = function()
                {
                    let labelsWithTag = {};
                    for (let [k, v] of Object.entries(localOptions.rekall().sortings.colors.categories)) {
                        for (let i in v.tags) {
                            v.tags[i].getLabels().forEach(l => {
                                if (!labelsWithTag[l]) {
                                    labelsWithTag[l] = [];
                                }
                                labelsWithTag[l].push(v.tags[i]);
                            });
                        }
                    }

                    return labelsWithTag;
                };
                function createViewHtmlFromLabels(labelsWithTag) {
                    for (let [label, v] of Object.entries(labelsWithTag)) {
                        let categoryName = label;
                        let category = $('<div/>').addClass('mosaic_category').css('background', rgbToHex(85, 146, 153));


                        let grid = $('<div/>').addClass('mosaic_category_grid');

                        category.append($('<h2/>').text(categoryName));
                        category.append(grid);

                        typefilterdiv.append(getFilterElement(
                            categoryName,
                            rgbToHex(85, 146, 153),
                            function () {
                                category.toggle();
                                $(this).toggleClass('mosaic_filter_disabled');
                                toggleLabelToShow(categoryName)
                            },
                            '',
                            localOptions.filter_label_to_hide.includes(categoryName)
                        ));

                        for (let i in v) {
                            grid.append(createHtmlElementForTagOrDoc(v[i]));
                        }

                        localOptions.htmlElement().append(category);

                        if (localOptions.filter_label_to_hide.includes(categoryName)){
                            category.toggle();
                        }
                    }
                };
                createViewHtmlFromLabels(groupTagsByLabels());
            }

            const fillPanel = function () {
                let resetfilterdiv = $('<div/>').addClass('mosaic_filter');
                let typefilterdiv = $('<div/>').addClass('mosaic_filter');
                let labelsfilterdiv = $('<div/>').addClass('mosaic_filter');
                resetfilterdiv.append(getFilterElement(
                    'Reset filters',
                    'rgb(100,100,100)',
                    localOptions._resetFilters,
                    'mosaic_filter_item_all',
                    false
                ));
                localOptions.htmlElement().append(createFilterTypeSelector());
                localOptions.htmlElement().append(resetfilterdiv);

                if (localOptions._filter_type === 0) {
                    localOptions.htmlElement().append(typefilterdiv);
                }
                if (localOptions._filter_type === 1){
                    localOptions.htmlElement().append(labelsfilterdiv);
                }
                // eslint-disable-next-line no-undef
                if (filterByCategory === localOptions._filter_type){
                    createViewByCategory(typefilterdiv);
                }
                if (filterByLabel === localOptions._filter_type){
                    createViewByLabel(labelsfilterdiv);
                }
            }

            localOptions.toggle_filterType = toggleFilterType;

            return {
                show: function () {
                    if (localOptions.isOpen)
                        return;

                    localOptions.rekall().timeline.pause();
                    fillPanel();
                    localOptions.htmlElement().show();
                    localOptions.updateObserver = localOptions.pubsub().subscribe('popupEdit.Closed', this.refresh);
                    localOptions.isOpen = true;
                },
                hide: function () {
                    if (!localOptions.isOpen)
                        return;

                    localOptions.pubsub().unsubscribe(localOptions.updateObserver);
                    localOptions.updateObserver = undefined;
                    localOptions.htmlElement().hide();
                    emptyPanel();
                    localOptions.isOpen = false;
                    localOptions._destroyFollowedElements();
                    localOptions.filters_type_to_hide = [];
                },
                refresh: refreshView
            };

        }

        app.rekall.Rekall('mosaicWidget', MosaicPanelWidget());

    }
)(window.app = window.app || {})
