<?php

namespace App\DataFixtures;

use App\Builder\UserBuilder;
use App\Entity\User;
use Doctrine\Bundle\FixturesBundle\Fixture;
use Doctrine\Persistence\ObjectManager;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;

class UserFixtures extends Fixture
{
    private UserPasswordHasherInterface $password_hasher;

    public function __construct(UserPasswordHasherInterface $password_hasher)
    {
        $this->password_hasher = $password_hasher;
    }

    public function load(ObjectManager $manager): void
    {
        $persistent_user_factory = $this->createUserPersistentFactory($manager);

        $persistent_user_factory(
            function (UserBuilder $builder) {
                return $builder->withEmail("notVerified@localhost.com")
                    ->withFirstName("Bob")
                    ->withLastName("Smith")
                    ->withPassword('', 'password')
                    ->withRoles([])
                    ->withIsVerified(false);
            }
        );

        $persistent_user_factory(
            function (UserBuilder $builder) {
                return $builder->withEmail("defaultUser@localhost.com")
                    ->withFirstName("Alice")
                    ->withLastName("Rango")
                    ->withPassword('', 'password')
                    ->withRoles([])
                    ->withIsVerified(true)
                    ->withAcceptGeneralConditions(true);
            }
        );

        $persistent_user_factory(
            function (UserBuilder $builder) {
                return $builder->withEmail("defaultUser2@localhost.com")
                    ->withFirstName("John")
                    ->withLastName("Doe")
                    ->withPassword("", 'password')
                    ->withRoles([])
                    ->withIsVerified(true);
            }
        );

        $manager->flush();
    }

    /**
     * Create a user using the callable parameter, save the result within the ObjectManage then return the created user.
     * @param ObjectManager $manager
     * @return callable function(callable $userFactory): void { ... }
     *                  <ul>
     *                     <li>The $userFactory should take one parameter of type UserBuilder</li>
     *                  </ul>
     */
    private function createUserPersistentFactory(ObjectManager $manager): callable
    {
        return function (callable $userBuilderAction) use ($manager): User {
            $userBuilder = new UserBuilder($this->password_hasher);
            $userBuilderAction($userBuilder);
            $newUser = $userBuilder->createUser();

            $manager->persist($newUser);

            return $newUser;
        };
    }
}
