<?php

namespace App\Tests\functional;

use App\Entity\User;
use Exception;
use Symfony\Bundle\FrameworkBundle\KernelBrowser;
use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;

class UserControllerTest extends WebTestCase
{
    private KernelBrowser $client;
    private User $verified_user;

    protected function setUp(): void
    {
        self::ensureKernelShutdown();
        $this->client = static::createClient();

        $object_manager = $this->client->getContainer()
            ->get('doctrine')
            ->getManager();

        $verified_user = $object_manager
            ->getRepository(User::class)
            ->findOneBy(['email' => 'defaultUser@localhost.com']);

        if (! $verified_user instanceof User) {
            throw new \Exception("User does not exist.");
        }

        $this->verified_user = $verified_user;
    }

    public function testWhenUserUpdateEmailHeReceivesAConfirmationLinkByEMail(): void
    {
        $new_email_address = 'defaultuser@localhost.test';
        $this->client->loginUser($this->verified_user);
        $crawler = $this->client->request('GET', '/en/edit_profile');

        $this->assertResponseIsSuccessful();
        $this->client->enableProfiler();

        $submit = $crawler->selectButton('Update');
        $form = $submit->form();
        $form['edit_user_profile_form[email]'] = $new_email_address;
        $form['edit_user_profile_form[current_password]'] = "password";
        $this->client->submit($form);

        $this->assertEmailCount(
            1,
            null,
            'Once the user update his email, the system should send him a confirmation link by email'
        );
        $emailMessage = $this->getMailerMessage(0);

        if (null === $emailMessage) {
            throw new Exception("Email message could not be found");
        }

        $this->assertEmailAddressContains(
            $emailMessage,
            'To',
            $new_email_address
        );
    }
}
